' This example illustrates how to configure a ZX to go to sleep and
' awaken on a pin change interrupt.  The example is hard-coded to
' awaken on a pin change on C.6 but it can be modified, with care, to
' use any other available pin.  With further changes, it can easily be
' modified to awaken on a specific edge of one of the external interrupts.
' Similar stragegies could be used to awaken on an analog compare event,
' Timer2 interrupt, etc.
'
' This code is specific to the ZX devices based on the mega644 and mega644P
' chips.  In most cases, it can be modified for other ZX devices after
' careful study of the respective datasheet.
'
' For mega chips that support it, power consumption can be further reduced
' while sleeping by setting one or more bits in the power reduction register.
'
' The example code employs two different strategies for implementing the
' wait-for-event.  The simplest strategy is to use a separate task that
' waits for the desired event.  This has the advantage of requiring fewer
' device-specific instructions for setup but has the disadvantage of consuming
' otherwise-unneeded RAM for the task's stack.  The alternate approach
' involves configuring the processor registers directly to generate an
' interrupt when the desired event occurs.  For VM mode devices, this works
' because the firmware already contains an interrupt handler for the
' designated interrupt.  For native mode devices, a simple stub interrupt
' handler would need to be provided.

' Uncomment the line below to use a task-based approach.  Otherwise,
' registers will be manipulated directly to achieve the awakening
' interrupt.
#define USE_TASK

Sub Main()
	Do
		' output a message, wait for it to complete transmission
		Debug.Print "Preparing to sleep."
		Do While CBool(StatusCom(1) And &H04)
		Loop

		' sleep until an awakening event occurs
		Call GotoSleep()

		' now awakened, display a message
		Debug.Print "Awakened from sleep."
	Loop
End Sub

Sub GotoSleep()
#ifdef USE_TASK
	Static taskRunning as Boolean
	Static ts1(1 to 50) as Byte

	' invoke the dummy task if it is not already running
	If Not taskRunning Then
		CallTask Task1, ts1
		taskRunning = true
	End If
	
	' ensure that the task is awaiting the pin change
	Do
		Call Sleep(0)
	Loop While StatusTask(ts1) <> TaskWaitPinChangeC
#else
	' prepare for an interrupt on pin change on C.6
	' N.B.: this code is specific to the mega644
	Register.PCMSK2 = &H40
	Call SetBits(Register.PCIFR, &H04, &H04)
	Call SetBits(Register.PCICR, &H04, &H04)
#endif

	' prepare for sleeping, awakening on a pin change
	' N.B.: this code is specific to the mega644
	Register.SMCR = &H04	' power-down mode
	Call CPUSleep()

#ifndef USE_TASK
	' disable the pin change interrupt
	' N.B.: this code is specific to the mega644
	Call SetBits(Register.PCICR, &H04, &H00)
#endif

End Sub

#ifdef USE_TASK

Sub Task1()
	Do
		' await a pin change on C.6
		Call WaitForInterrupt(&H40, WaitPinChangeC)
	Loop
End Sub

#endif


